import turbogears
from model import Page, hub
from docutils.core import publish_parts
import cherrypy
import re
from sqlobject import SQLObjectNotFound
from turbogears import validators

wikiwords = re.compile(r"\b([A-Z]\w+[A-Z]\w+)")

class Root:
    @turbogears.expose(html="wiki20.templates.page")
    def default(self, pagename):
        return self.index(pagename)
    
    @turbogears.expose(html="wiki20.templates.page")
    def index(self, pagename="FrontPage"):
        try:
            page = Page.byPagename(pagename)
        except SQLObjectNotFound:
            raise cherrypy.HTTPRedirect("/notfound?pagename=%s" % pagename)
        content = publish_parts(page.data,
            writer_name="html")["html_body"]
        content = wikiwords.sub(r'<a href="/\1">\1</a>', content)
        return dict(pagename=pagename, data=content)
        
    @turbogears.expose(html="wiki20.templates.edit")
    def edit(self, pagename):
        page = Page.byPagename(pagename)
        return dict(pagename=pagename, data=page.data, new=False)
    
    @turbogears.expose(validators=dict(
                        new=validators.StringBoolean()))
    def save(self, pagename, data, submit, new):
        hub.begin()
        if new:
            page = Page(pagename=pagename, data=data)
        else:
            page = Page.byPagename(pagename)
            page.data = data
        hub.commit()
        hub.end()
        turbogears.flash("Changes saved!")
        raise cherrypy.HTTPRedirect("/%s" % pagename)

    @turbogears.expose(html="wiki20.templates.edit")
    def notfound(self, pagename):
        return dict(pagename=pagename, data="", new=True)
    
    @turbogears.expose(html="wiki20.templates.pagelist")
    def pagelist(self):
        pages = [page.pagename for page in
            Page.select(orderBy=Page.q.pagename)]
        return dict(pages=pages)